'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.raw = undefined;
exports.default = loader;

var _loaderUtils = require('loader-utils');

var _schemaUtils = require('schema-utils');

var _schemaUtils2 = _interopRequireDefault(_schemaUtils);

var _mime = require('mime');

var _mime2 = _interopRequireDefault(_mime);

var _normalizeFallback2 = require('./utils/normalizeFallback');

var _normalizeFallback3 = _interopRequireDefault(_normalizeFallback2);

var _options = require('./options.json');

var _options2 = _interopRequireDefault(_options);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function shouldTransform(limit, size) {
  if (typeof limit === 'boolean') {
    return limit;
  }

  if (typeof limit === 'string') {
    return size <= parseInt(limit, 10);
  }

  if (typeof limit === 'number') {
    return size <= limit;
  }

  return true;
}

function loader(src) {
  // Loader Options
  var options = (0, _loaderUtils.getOptions)(this) || {};

  (0, _schemaUtils2.default)(_options2.default, options, {
    name: 'URL Loader',
    baseDataPath: 'options'
  });

  // No limit or within the specified limit
  if (shouldTransform(options.limit, src.length)) {
    var file = this.resourcePath;
    var mimetype = options.mimetype || _mime2.default.getType(file);

    if (typeof src === 'string') {
      // eslint-disable-next-line no-param-reassign
      src = Buffer.from(src);
    }

    var esModule = typeof options.esModule !== 'undefined' ? options.esModule : true;

    return (esModule ? 'export default' : 'module.exports =') + ' ' + JSON.stringify('data:' + (mimetype || '') + ';base64,' + src.toString('base64'));
  }

  // Normalize the fallback.

  var _normalizeFallback = (0, _normalizeFallback3.default)(options.fallback, options),
      fallbackLoader = _normalizeFallback.loader,
      fallbackOptions = _normalizeFallback.options;

  // Require the fallback.
  // eslint-disable-next-line global-require, import/no-dynamic-require


  var fallback = require(fallbackLoader);

  // Call the fallback, passing a copy of the loader context. The copy has the query replaced. This way, the fallback
  // loader receives the query which was intended for it instead of the query which was intended for url-loader.
  var fallbackLoaderContext = Object.assign({}, this, {
    query: fallbackOptions
  });

  return fallback.call(fallbackLoaderContext, src);
}

// Loader Mode
var raw = exports.raw = true;